@@ -127,7 +127,7 @@ void QFSFileEnginePrivate::init()
     is_link = 0;
     openMode = QIODevice::NotOpen;
     fd = -1;
-    fh = 0;
+    fh = nullptr;
     lastIOCommand = IOFlushCommand;
     lastFlushFailed = false;
     closeFileHandle = false;
@@ -247,7 +247,7 @@ bool QFSFileEngine::open(QIODevice::OpenMode openMode)
     d->openMode = res.openMode;
     d->lastFlushFailed = false;
     d->tried_stat = 0;
-    d->fh = 0;
+    d->fh = nullptr;
     d->fd = -1;
 
     return d->nativeOpen(d->openMode);
@@ -309,7 +309,7 @@ bool QFSFileEnginePrivate::openFh(QIODevice::OpenMode openMode, FILE *fh)
                         QSystemError::stdString());
 
             this->openMode = QIODevice::NotOpen;
-            this->fh = 0;
+            this->fh = nullptr;
 
             return false;
         }
@@ -341,7 +341,7 @@ bool QFSFileEngine::open(QIODevice::OpenMode openMode, int fd, QFile::FileHandle
     d->lastFlushFailed = false;
     d->closeFileHandle = (handleFlags & QFile::AutoCloseHandle);
     d->fileEntry.clear();
-    d->fh = 0;
+    d->fh = nullptr;
     d->fd = -1;
     d->tried_stat = 0;
 
@@ -357,11 +357,11 @@ bool QFSFileEnginePrivate::openFd(QIODevice::OpenMode openMode, int fd)
 {
     Q_Q(QFSFileEngine);
     this->fd = fd;
-    fh = 0;
+    fh = nullptr;
 
     // Seek to the end when in Append mode.
     if (openMode & QFile::Append) {
-        int ret;
+        QT_OFF_T ret;
         do {
             ret = QT_LSEEK(fd, 0, SEEK_END);
         } while (ret == -1 && errno == EINTR);
@@ -418,7 +418,7 @@ bool QFSFileEnginePrivate::closeFdFh()
 
         // We must reset these guys regardless; calling close again after a
         // failed close causes crashes on some systems.
-        fh = 0;
+        fh = nullptr;
         fd = -1;
         closed = (ret == 0);
     }
@@ -835,7 +835,7 @@ QAbstractFileEngine::Iterator *QFSFileEngine::beginEntryList(QDir::Filters filte
 */
 QAbstractFileEngine::Iterator *QFSFileEngine::endEntryList()
 {
-    return 0;
+    return nullptr;
 }
 #endif // QT_NO_FILESYSTEMITERATOR
 
@@ -883,7 +883,7 @@ bool QFSFileEngine::extension(Extension extension, const ExtensionOption *option
         const MapExtensionOption *options = (const MapExtensionOption*)(option);
         MapExtensionReturn *returnValue = static_cast<MapExtensionReturn*>(output);
         returnValue->address = d->map(options->offset, options->size, options->flags);
-        return (returnValue->address != 0);
+        return (returnValue->address != nullptr);
     }
     if (extension == UnMapExtension) {
         const UnMapExtensionOption *options = (const UnMapExtensionOption*)option;
@@ -911,14 +911,7 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
 }
 
 /*! \fn bool QFSFileEngine::caseSensitive() const
-  Returns \c true for Windows, false for Unix.
-*/
-
-/*! \fn bool QFSFileEngine::copy(const QString &copyName)
-
-  For Windows or Apple platforms, copy the file to file \a copyName.
-
-  Not implemented for other Unix platforms.
+  Returns \c false for Windows, true for Unix.
 */
 
 /*! \fn QString QFSFileEngine::currentPath(const QString &fileName)
@@ -950,11 +943,34 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
   \reimp
 */
 
-/*! \fn QString QFSFileEngine::homePath()
+/*!
   Returns the home path of the current user.
 
   \sa rootPath()
 */
+QString QFSFileEngine::homePath()
+{
+    return QFileSystemEngine::homePath();
+}
+
+/*!
+  Returns the root path.
+
+  \sa homePath()
+*/
+QString QFSFileEngine::rootPath()
+{
+    return QFileSystemEngine::rootPath();
+}
+
+/*!
+  Returns the temporary path (i.e., a path in which it is safe
+  to store temporary files).
+*/
+QString QFSFileEngine::tempPath()
+{
+    return QFileSystemEngine::tempPath();
+}
 
 /*! \fn bool QFSFileEngine::isRelativePath() const
   \reimp
@@ -968,9 +984,6 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
   true if successful; otherwise returns \c false.
 */
 
-/*! \fn bool QFSFileEngine::mkdir(const QString &name, bool createParentDirectories) const
-  \reimp
-*/
 
 /*! \fn uint QFSFileEngine::ownerId(QAbstractFileEngine::FileOwner own) const
   In Unix, if stat() is successful, the \c uid is returned if
@@ -984,35 +997,87 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
   \reimp
 */
 
-/*! \fn bool QFSFileEngine::remove()
+/*!
+  For Windows or Apple platforms, copy the file to file \a copyName.
+
+  Not implemented for other Unix platforms.
+*/
+bool QFSFileEngine::copy(const QString &copyName)
+{
+    Q_D(QFSFileEngine);
+    QSystemError error;
+    bool ret = QFileSystemEngine::copyFile(d->fileEntry, QFileSystemEntry(copyName), error);
+    if (!ret)
+        setError(QFile::CopyError, error.toString());
+    return ret;
+}
+
+/*!
   \reimp
 */
+bool QFSFileEngine::remove()
+{
+    Q_D(QFSFileEngine);
+    QSystemError error;
+    bool ret = QFileSystemEngine::removeFile(d->fileEntry, error);
+    d->metaData.clear();
+    if (!ret)
+        setError(QFile::RemoveError, error.toString());
+    return ret;
+}
 
-/*! \fn bool QFSFileEngine::rename(const QString &newName)
+/*!
   \reimp
 */
-
-
-/*! \fn bool QFSFileEngine::renameOverwrite(const QString &newName)
+bool QFSFileEngine::rename(const QString &newName)
+{
+    Q_D(QFSFileEngine);
+    QSystemError error;
+    bool ret = QFileSystemEngine::renameFile(d->fileEntry, QFileSystemEntry(newName), error);
+    if (!ret)
+        setError(QFile::RenameError, error.toString());
+    return ret;
+}
+/*!
   \reimp
 */
+bool QFSFileEngine::renameOverwrite(const QString &newName)
+{
+    Q_D(QFSFileEngine);
+    QSystemError error;
+    bool ret = QFileSystemEngine::renameOverwriteFile(d->fileEntry, QFileSystemEntry(newName), error);
+    if (!ret)
+        setError(QFile::RenameError, error.toString());
+    return ret;
+}
 
-/*! \fn bool QFSFileEngine::rmdir(const QString &name, bool recurseParentDirectories) const
+/*!
   \reimp
 */
+bool QFSFileEngine::mkdir(const QString &name, bool createParentDirectories) const
+{
+    return QFileSystemEngine::createDirectory(QFileSystemEntry(name), createParentDirectories);
+}
 
-/*! \fn QString QFSFileEngine::rootPath()
-  Returns the root path.
-
-  \sa homePath()
+/*!
+  \reimp
 */
+bool QFSFileEngine::rmdir(const QString &name, bool recurseParentDirectories) const
+{
+    return QFileSystemEngine::removeDirectory(QFileSystemEntry(name), recurseParentDirectories);
+}
 
-/*! \fn bool QFSFileEngine::setCurrentPath(const QString &path)
+
+/*!
   Sets the current path (e.g., for QDir), to \a path. Returns \c true if the
   new path exists; otherwise this function does nothing, and returns \c false.
 
   \sa currentPath()
 */
+bool QFSFileEngine::setCurrentPath(const QString &path)
+{
+    return QFileSystemEngine::setCurrentPath(QFileSystemEntry(path));
+}
 
 /*! \fn bool QFSFileEngine::setPermissions(uint perms)
   \reimp
@@ -1022,11 +1087,6 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
   \reimp
 */
 
-/*! \fn QString QFSFileEngine::tempPath()
-  Returns the temporary path (i.e., a path in which it is safe
-  to store temporary files).
-*/
-
 /*! \fn QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions(QAbstractFileEngine::FileFlags type) const
     \internal
 */
