@@ -275,7 +275,7 @@ QT_BEGIN_NAMESPACE
          \li "<APPROOT>/files"
          \li "<APPROOT>/Documents/Desktop"
     \row \li DocumentsLocation
-         \li "<USER>/Documents", "<USER>/<APPNAME>/Documents"
+         \li "<USER>/Documents" [*], "<USER>/<APPNAME>/Documents"
          \li "<APPROOT>/Documents"
     \row \li FontsLocation
          \li "/system/fonts" (not writable)
@@ -284,13 +284,13 @@ QT_BEGIN_NAMESPACE
          \li not supported (directory not readable)
          \li not supported
     \row \li MusicLocation
-         \li "<USER>/Music", "<USER>/<APPNAME>/Music"
+         \li "<USER>/Music" [*], "<USER>/<APPNAME>/Music"
          \li "<APPROOT>/Documents/Music"
     \row \li MoviesLocation
-         \li "<USER>/Movies", "<USER>/<APPNAME>/Movies"
+         \li "<USER>/Movies" [*], "<USER>/<APPNAME>/Movies"
          \li "<APPROOT>/Documents/Movies"
     \row \li PicturesLocation
-         \li "<USER>/Pictures", "<USER>/<APPNAME>/Pictures"
+         \li "<USER>/Pictures" [*], "<USER>/<APPNAME>/Pictures"
          \li "<APPROOT>/Documents/Pictures", "assets-library://"
     \row \li TempLocation
          \li "<APPROOT>/cache"
@@ -305,8 +305,8 @@ QT_BEGIN_NAMESPACE
          \li "<APPROOT>/cache", "<USER>/<APPNAME>/cache"
          \li "<APPROOT>/Library/Caches"
     \row \li GenericDataLocation
-         \li "<USER>"
-         \li "<APPROOT>/Documents"
+         \li "<USER>" [*] or "<USER>/<APPNAME>/files"
+         \li "<APPROOT>/Library/Application Support"
     \row \li RuntimeLocation
          \li "<APPROOT>/cache"
          \li not supported
@@ -317,7 +317,7 @@ QT_BEGIN_NAMESPACE
          \li "<APPROOT>/files/settings" (there is no shared settings)
          \li "<APPROOT>/Library/Preferences"
     \row \li DownloadLocation
-         \li "<USER>/Downloads", "<USER>/<APPNAME>/Downloads"
+         \li "<USER>/Downloads" [*], "<USER>/<APPNAME>/Downloads"
          \li "<APPROOT>/Documents/Downloads"
     \row \li GenericCacheLocation
          \li "<APPROOT>/cache" (there is no shared cache)
@@ -348,6 +348,11 @@ QT_BEGIN_NAMESPACE
     \note On Android 6.0 (API 23) or higher, the "WRITE_EXTERNAL_STORAGE" permission must be
         requested at runtime when using QStandardPaths::writableLocation or QStandardPaths::standardLocations.
 
+    \note [*] On Android 11 and above, public directories are no longer directly accessible
+    in scoped storage mode. Thus, paths of the form \c "<USER>/DirName" are not returned.
+    Instead, you can use \l QFileDialog which uses the Storage Access Framework (SAF)
+    to access such directories.
+
     \note On iOS, if you do pass \c {QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).last()}
         as argument to \l{QFileDialog::setDirectory()},
         a native image picker dialog will be used for accessing the user's photo album.
