@@ -70,7 +70,7 @@ class QRecursiveMutexPrivate : public QMutexData
 {
 public:
     QRecursiveMutexPrivate()
-        : QMutexData(QMutex::Recursive), owner(0), count(0) {}
+        : QMutexData(QMutex::Recursive), owner(nullptr), count(0) {}
 
     // written to by the thread that first owns 'mutex';
     // read during attempts to acquire ownership of 'mutex' from any other thread:
@@ -82,7 +82,7 @@ public:
     QMutex mutex;
 
     bool lock(int timeout) QT_MUTEX_LOCK_NOEXCEPT;
-    void unlock() Q_DECL_NOTHROW;
+    void unlock() noexcept;
 };
 
 /*
@@ -147,7 +147,7 @@ public:
     It is constructed and destroyed with almost no overhead,
     which means it is fine to have many mutexes as part of other classes.
 
-    \sa QMutexLocker, QReadWriteLock, QSemaphore, QWaitCondition
+    \sa QRecursiveMutex, QMutexLocker, QReadWriteLock, QSemaphore, QWaitCondition
 */
 
 /*!
@@ -156,12 +156,19 @@ public:
     \value Recursive  In this mode, a thread can lock the same mutex
                       multiple times and the mutex won't be unlocked
                       until a corresponding number of unlock() calls
-                      have been made.
+                      have been made. You should use QRecursiveMutex
+                      for this use-case.
 
     \value NonRecursive  In this mode, a thread may only lock a mutex
                          once.
 
-    \sa QMutex()
+    \sa QMutex(), QRecursiveMutex
+*/
+
+/*!
+    \fn QMutex::QMutex()
+
+    Constructs a new mutex. The mutex is created in an unlocked state.
 */
 
 /*!
@@ -179,7 +186,7 @@ public:
 */
 QMutex::QMutex(RecursionMode mode)
 {
-    d_ptr.store(mode == Recursive ? new QRecursiveMutexPrivate : 0);
+    d_ptr.storeRelaxed(mode == Recursive ? new QRecursiveMutexPrivate : nullptr);
 }
 
 /*!
@@ -189,12 +196,12 @@ QMutex::QMutex(RecursionMode mode)
 */
 QMutex::~QMutex()
 {
-    QMutexData *d = d_ptr.load();
+    QMutexData *d = d_ptr.loadRelaxed();
     if (isRecursive()) {
         delete static_cast<QRecursiveMutexPrivate *>(d);
     } else if (d) {
 #ifndef QT_LINUX_FUTEX
-        if (d != dummyLocked() && static_cast<QMutexPrivate *>(d)->possiblyUnlocked.load()
+        if (d != dummyLocked() && static_cast<QMutexPrivate *>(d)->possiblyUnlocked.loadRelaxed()
             && tryLock()) {
             unlock();
             return;
@@ -205,13 +212,14 @@ QMutex::~QMutex()
 }
 
 /*! \fn void QMutex::lock()
+
     Locks the mutex. If another thread has locked the mutex then this
     call will block until that thread has unlocked it.
 
     Calling this function multiple times on the same mutex from the
     same thread is allowed if this mutex is a
-    \l{QMutex::Recursive}{recursive mutex}. If this mutex is a
-    \l{QMutex::NonRecursive}{non-recursive mutex}, this function will
+    \l{QRecursiveMutex}{recursive mutex}. If this mutex is a
+    \l{QMutex}{non-recursive mutex}, this function will
     \e dead-lock when the mutex is locked recursively.
 
     \sa unlock()
@@ -243,8 +251,8 @@ void QMutex::lock() QT_MUTEX_LOCK_NOEXCEPT
 
     Calling this function multiple times on the same mutex from the
     same thread is allowed if this mutex is a
-    \l{QMutex::Recursive}{recursive mutex}. If this mutex is a
-    \l{QMutex::NonRecursive}{non-recursive mutex}, this function will
+    \l{QRecursiveMutex}{recursive mutex}. If this mutex is a
+    \l{QMutex}{non-recursive mutex}, this function will
     \e always return false when attempting to lock the mutex
     recursively.
 
@@ -290,8 +298,8 @@ bool QMutex::tryLock(int timeout) QT_MUTEX_LOCK_NOEXCEPT
 
     Calling this function multiple times on the same mutex from the
     same thread is allowed if this mutex is a
-    \l{QMutex::Recursive}{recursive mutex}. If this mutex is a
-    \l{QMutex::NonRecursive}{non-recursive mutex}, this function will
+    \l{QRecursiveMutex}{recursive mutex}. If this mutex is a
+    \l{QMutex}{non-recursive mutex}, this function will
     \e always return false when attempting to lock the mutex
     recursively.
 
@@ -314,8 +322,8 @@ bool QMutex::tryLock(int timeout) QT_MUTEX_LOCK_NOEXCEPT
 
     Calling this function multiple times on the same mutex from the
     same thread is allowed if this mutex is a
-    \l{QMutex::Recursive}{recursive mutex}. If this mutex is a
-    \l{QMutex::NonRecursive}{non-recursive mutex}, this function will
+    \l{QRecursiveMutex}{recursive mutex}. If this mutex is a
+    \l{QMutex}{non-recursive mutex}, this function will
     \e always return false when attempting to lock the mutex
     recursively.
 
@@ -323,13 +331,14 @@ bool QMutex::tryLock(int timeout) QT_MUTEX_LOCK_NOEXCEPT
 */
 
 /*! \fn void QMutex::unlock()
+
     Unlocks the mutex. Attempting to unlock a mutex in a different
     thread to the one that locked it results in an error. Unlocking a
     mutex that is not locked results in undefined behavior.
 
     \sa lock()
 */
-void QMutex::unlock() Q_DECL_NOTHROW
+void QMutex::unlock() noexcept
 {
     QMutexData *current;
     if (fastTryUnlock(current))
@@ -348,7 +357,7 @@ void QMutex::unlock() Q_DECL_NOTHROW
     Returns \c true if the mutex is recursive.
 */
 
-bool QBasicMutex::isRecursive() Q_DECL_NOTHROW
+bool QBasicMutex::isRecursive() noexcept
 {
     return QT_PREPEND_NAMESPACE(isRecursive)(d_ptr.loadAcquire());
 }
@@ -358,11 +367,63 @@ bool QBasicMutex::isRecursive() Q_DECL_NOTHROW
 
     Returns \c true if the mutex is recursive.
 */
-bool QBasicMutex::isRecursive() const Q_DECL_NOTHROW
+bool QBasicMutex::isRecursive() const noexcept
 {
     return QT_PREPEND_NAMESPACE(isRecursive)(d_ptr.loadAcquire());
 }
 
+/*!
+    \class QRecursiveMutex
+    \inmodule QtCore
+    \since 5.14
+    \brief The QRecursiveMutex class provides access serialization between threads.
+
+    \threadsafe
+
+    \ingroup thread
+
+    The QRecursiveMutex class is a mutex, like QMutex, with which it is
+    API-compatible. It differs from QMutex by accepting lock() calls from
+    the same thread any number of times. QMutex would deadlock in this situation.
+
+    QRecursiveMutex is much more expensive to construct and operate on, so
+    use a plain QMutex whenever you can. Sometimes, one public function,
+    however, calls another public function, and they both need to lock the
+    same mutex. In this case, you have two options:
+
+    \list
+    \li Factor the code that needs mutex protection into private functions,
+    which assume that the mutex is held when they are called, and lock a
+    plain QMutex in the public functions before you call the private
+    implementation ones.
+    \li Or use a recursive mutex, so it doesn't matter that the first public
+    function has already locked the mutex when the second one wishes to do so.
+    \endlist
+
+    \sa QMutex, QMutexLocker, QReadWriteLock, QSemaphore, QWaitCondition
+*/
+
+/*!
+    Constructs a new recursive mutex. The mutex is created in an unlocked state.
+
+    \sa lock(), unlock()
+*/
+QRecursiveMutex::QRecursiveMutex()
+    : QMutex()
+{
+    d_ptr.storeRelaxed(new QRecursiveMutexPrivate);
+}
+
+/*!
+    Destroys the mutex.
+
+    \warning Destroying a locked mutex may result in undefined behavior.
+*/
+QRecursiveMutex::~QRecursiveMutex()
+{
+    delete static_cast<QRecursiveMutexPrivate*>(d_ptr.fetchAndStoreAcquire(nullptr));
+}
+
 /*!
     \class QMutexLocker
     \inmodule QtCore
@@ -420,12 +481,23 @@ bool QBasicMutex::isRecursive() const Q_DECL_NOTHROW
     \fn QMutexLocker::QMutexLocker(QMutex *mutex)
 
     Constructs a QMutexLocker and locks \a mutex. The mutex will be
-    unlocked when the QMutexLocker is destroyed. If \a mutex is zero,
+    unlocked when the QMutexLocker is destroyed. If \a mutex is \nullptr,
     QMutexLocker does nothing.
 
     \sa QMutex::lock()
 */
 
+/*!
+    \fn QMutexLocker::QMutexLocker(QRecursiveMutex *mutex)
+    \since 5.14
+
+    Constructs a QMutexLocker and locks \a mutex. The mutex will be
+    unlocked (unlock() called) when the QMutexLocker is destroyed.
+    If \a mutex is \nullptr, QMutexLocker does nothing.
+
+    \sa QMutex::lock()
+*/
+
 /*!
     \fn QMutexLocker::~QMutexLocker()
 
@@ -517,7 +589,7 @@ bool QBasicMutex::lockInternal(int timeout) QT_MUTEX_LOCK_NOEXCEPT
         }
 
         QMutexPrivate *d = static_cast<QMutexPrivate *>(copy);
-        if (timeout == 0 && !d->possiblyUnlocked.load())
+        if (timeout == 0 && !d->possiblyUnlocked.loadRelaxed())
             return false;
 
         // At this point we have a pointer to a QMutexPrivate. But the other thread
@@ -541,7 +613,7 @@ bool QBasicMutex::lockInternal(int timeout) QT_MUTEX_LOCK_NOEXCEPT
         // is set to the BigNumber magic value set in unlockInternal()
         int old_waiters;
         do {
-            old_waiters = d->waiters.load();
+            old_waiters = d->waiters.loadAcquire();
             if (old_waiters == -QMutexPrivate::BigNumber) {
                 // we are unlocking, and the thread that unlocks is about to change d to 0
                 // we try to acquire the mutex by changing to dummyLocked()
@@ -550,7 +622,7 @@ bool QBasicMutex::lockInternal(int timeout) QT_MUTEX_LOCK_NOEXCEPT
                     d->deref();
                     return true;
                 } else {
-                    Q_ASSERT(d != d_ptr.load()); //else testAndSetAcquire should have succeeded
+                    Q_ASSERT(d != d_ptr.loadRelaxed()); //else testAndSetAcquire should have succeeded
                     // Mutex is likely to bo 0, we should continue the outer-loop,
                     //  set old_waiters to the magic value of BigNumber
                     old_waiters = QMutexPrivate::BigNumber;
@@ -563,7 +635,7 @@ bool QBasicMutex::lockInternal(int timeout) QT_MUTEX_LOCK_NOEXCEPT
             // The mutex was unlocked before we incremented waiters.
             if (old_waiters != QMutexPrivate::BigNumber) {
                 //we did not break the previous loop
-                Q_ASSERT(d->waiters.load() >= 1);
+                Q_ASSERT(d->waiters.loadRelaxed() >= 1);
                 d->waiters.deref();
             }
             d->deref();
@@ -572,11 +644,11 @@ bool QBasicMutex::lockInternal(int timeout) QT_MUTEX_LOCK_NOEXCEPT
 
         if (d->wait(timeout)) {
             // reset the possiblyUnlocked flag if needed (and deref its corresponding reference)
-            if (d->possiblyUnlocked.load() && d->possiblyUnlocked.testAndSetRelaxed(true, false))
+            if (d->possiblyUnlocked.loadRelaxed() && d->possiblyUnlocked.testAndSetRelaxed(true, false))
                 d->deref();
             d->derefWaiters(1);
             //we got the lock. (do not deref)
-            Q_ASSERT(d == d_ptr.load());
+            Q_ASSERT(d == d_ptr.loadRelaxed());
             return true;
         } else {
             Q_ASSERT(timeout >= 0);
@@ -593,14 +665,14 @@ bool QBasicMutex::lockInternal(int timeout) QT_MUTEX_LOCK_NOEXCEPT
             return false;
         }
     }
-    Q_ASSERT(d_ptr.load() != 0);
+    Q_ASSERT(d_ptr.loadRelaxed() != 0);
     return true;
 }
 
 /*!
     \internal
 */
-void QBasicMutex::unlockInternal() Q_DECL_NOTHROW
+void QBasicMutex::unlockInternal() noexcept
 {
     QMutexData *copy = d_ptr.loadAcquire();
     Q_ASSERT(copy); //we must be locked
@@ -618,7 +690,7 @@ void QBasicMutex::unlockInternal() Q_DECL_NOTHROW
         //there is no one waiting on this mutex anymore, set the mutex as unlocked (d = 0)
         if (d_ptr.testAndSetRelease(d, 0)) {
             // reset the possiblyUnlocked flag if needed (and deref its corresponding reference)
-            if (d->possiblyUnlocked.load() && d->possiblyUnlocked.testAndSetRelaxed(true, false))
+            if (d->possiblyUnlocked.loadRelaxed() && d->possiblyUnlocked.testAndSetRelaxed(true, false))
                 d->deref();
         }
         d->derefWaiters(0);
@@ -657,30 +729,30 @@ QMutexPrivate *QMutexPrivate::allocate()
     int i = freelist()->next();
     QMutexPrivate *d = &(*freelist())[i];
     d->id = i;
-    Q_ASSERT(d->refCount.load() == 0);
+    Q_ASSERT(d->refCount.loadRelaxed() == 0);
     Q_ASSERT(!d->recursive);
-    Q_ASSERT(!d->possiblyUnlocked.load());
-    Q_ASSERT(d->waiters.load() == 0);
-    d->refCount.store(1);
+    Q_ASSERT(!d->possiblyUnlocked.loadRelaxed());
+    Q_ASSERT(d->waiters.loadRelaxed() == 0);
+    d->refCount.storeRelaxed(1);
     return d;
 }
 
 void QMutexPrivate::release()
 {
     Q_ASSERT(!recursive);
-    Q_ASSERT(refCount.load() == 0);
-    Q_ASSERT(!possiblyUnlocked.load());
-    Q_ASSERT(waiters.load() == 0);
+    Q_ASSERT(refCount.loadRelaxed() == 0);
+    Q_ASSERT(!possiblyUnlocked.loadRelaxed());
+    Q_ASSERT(waiters.loadRelaxed() == 0);
     freelist()->release(id);
 }
 
 // atomically subtract "value" to the waiters, and remove the QMutexPrivate::BigNumber flag
-void QMutexPrivate::derefWaiters(int value) Q_DECL_NOTHROW
+void QMutexPrivate::derefWaiters(int value) noexcept
 {
     int old_waiters;
     int new_waiters;
     do {
-        old_waiters = waiters.load();
+        old_waiters = waiters.loadRelaxed();
         new_waiters = old_waiters;
         if (new_waiters < 0) {
             new_waiters += QMutexPrivate::BigNumber;
@@ -696,7 +768,7 @@ void QMutexPrivate::derefWaiters(int value) Q_DECL_NOTHROW
 inline bool QRecursiveMutexPrivate::lock(int timeout) QT_MUTEX_LOCK_NOEXCEPT
 {
     Qt::HANDLE self = QThread::currentThreadId();
-    if (owner.load() == self) {
+    if (owner.loadRelaxed() == self) {
         ++count;
         Q_ASSERT_X(count != 0, "QMutex::lock", "Overflow in recursion counter");
         return true;
@@ -709,19 +781,19 @@ inline bool QRecursiveMutexPrivate::lock(int timeout) QT_MUTEX_LOCK_NOEXCEPT
     }
 
     if (success)
-        owner.store(self);
+        owner.storeRelaxed(self);
     return success;
 }
 
 /*!
    \internal
  */
-inline void QRecursiveMutexPrivate::unlock() Q_DECL_NOTHROW
+inline void QRecursiveMutexPrivate::unlock() noexcept
 {
     if (count > 0) {
         count--;
     } else {
-        owner.store(0);
+        owner.storeRelaxed(nullptr);
         mutex.QBasicMutex::unlock();
     }
 }
