@@ -40,6 +40,8 @@
 
 #include <qcryptographichash.h>
 #include <qiodevice.h>
+#include <qmutex.h>
+#include <private/qlocking_p.h>
 
 #include "../../3rdparty/sha1/sha1.cpp"
 
@@ -58,6 +60,10 @@ typedef unsigned char BitSequence;
 typedef unsigned long long DataLength;
 typedef enum { SUCCESS = 0, FAIL = 1, BAD_HASHLEN = 2 } HashReturn;
 
+#ifdef Q_OS_RTEMS
+#  undef ALIGN
+#endif
+
 #include "../../3rdparty/sha3/KeccakSponge.c"
 typedef spongeState hashState;
 
@@ -188,6 +194,8 @@ public:
     };
     void sha3Finish(int bitCount, Sha3Variant sha3Variant);
 #endif
+    // protects result in result()
+    QMutex finalizeMutex;
     QByteArray result;
 };
 
@@ -444,6 +452,9 @@ bool QCryptographicHash::addData(QIODevice* device)
 */
 QByteArray QCryptographicHash::result() const
 {
+    // result() is a const function, so concurrent calls are allowed; protect:
+    const auto lock = qt_scoped_lock(d->finalizeMutex);
+    // check that no other thread already finalized before us:
     if (!d->result.isEmpty())
         return d->result;
 
@@ -531,6 +542,7 @@ QByteArray QCryptographicHash::result() const
     }
 #endif
     }
+
     return d->result;
 }
 
